# autogenerated by ssc-gen DO NOT_EDIT
from __future__ import annotations
import re
import json
from typing import TypedDict, Union
import sys

if sys.version_info >= (3, 10):
    from types import NoneType
else:
    NoneType = type(None)

from parsel import Selector
from parsel.selector import _SelectorType  # noqa

J_Dash = TypedDict(
    "J_Dash",
    {
        "src": str,
        "type": str,
    },
)
J_Hls = TypedDict(
    "J_Hls",
    {
        "src": str,
        "type": str,
    },
)
J_DataParameters = TypedDict(
    "J_DataParameters",
    {
        "id": str,
        "error": str,
        "domain": str,
        "cdn": str,
        "counter": str,
        "duration": str,
        "poster": str,
        "thumbnails": str,
        "dash": J_Dash,
        "hls": J_Hls,
        "quality": bool,
        "qualityVideo": int,
        "vast": bool,
        "country": str,
        "platform": str,
        "rating": str,
        "nshowbl": bool,
        "limitRate": bool,
        "aBlocklimitRate": bool,
    },
)
T_AniboomPage = TypedDict(
    "T_AniboomPage",
    {
        "data_parameters": J_DataParameters,
        "hls": str,
        "dash": str,
    },
)


class AniboomPage:
    """Extract MPD and M3U8 urls

        Required `referer="https://animego.org/` HEADER (.me, .club?)

        USAGE:
            1. GET <PLAYER_LINK> (e.g. https://aniboom.one/embed/6BmMbB7MxWO?episode=1&translation=30)
            2. PARSE. If pre-unescape response before parse - css selector may not find attribute
            3. For video playing, url required next headers:

            - Referer="https://aniboom.one/"
            - Accept-Language="ru-RU"  # INCREASE DOWNLOAD SPEED with this static value
            - Origin="https://aniboom.one"
        ISSUES:
            - 403 Forbidden if request sent not from CIS region
            - KEYS SHOULD BE STARTED IN Title Case else hls/mpd links returns 403 error
            - Sometimes, aniboom backend missing MPD key and returns M3U8 url. Check this value before usage:

            https://github.com/vypivshiy/ani-cli-ru/issues/29

            Expected json signature (LOOK at dash.src and hls.src keys):

            { ...
            "dash":"{"src":"https:.../abcdef.mpd",        "type":"application\\/dash+xml"}",
            "hls":"{"src":"https:...\\/master_device.m3u8",
            "type":"application\\/x-mpegURL"}"

            ... }

            MAYBE returns this:

             { ...
            "dash":"{"src":"https:...master_device.m3u8",        "type":"application\\/dash+xml"}",
            "hls":"{"src":"https:...master_device.m3u8",
            "type":"application\\/x-mpegURL"}"

            ... }





    {
        "data_parameters": {
            "id": "String",
            "error": "String",
            "domain": "String",
            "cdn": "String",
            "counter": "String",
            "duration": "String",
            "poster": "String",
            "thumbnails": "String",
            "dash": {
                "src": "String",
                "type": "String"
            },
            "hls": {
                "src": "String",
                "type": "String"
            },
            "quality": "Bool",
            "qualityVideo": "Int",
            "vast": "Bool",
            "country": "String",
            "platform": "String",
            "rating": "String",
            "nshowbl": "Bool",
            "limitRate": "Bool",
            "aBlocklimitRate": "Bool"
        },
        "hls": "String",
        "dash": "String"
    }"""

    def __init__(self, document: Union[str, _SelectorType]) -> None:
        self._doc = Selector(document) if isinstance(document, str) else document

    def _parse_data_parameters(self, value: Selector) -> J_DataParameters:
        value1 = value.css("#video")
        value2 = value1.attrib["data-parameters"]
        value3 = value2.replace("\\", "")
        value4 = value3.replace("&quot;}", "}")
        value5 = value4.replace("&quot;{", "{")
        value6 = value5.replace("&quot;", '"')
        value7 = value6.replace('}"', "}")
        value8 = value7.replace('"{', "{")
        return json.loads(value8)

    def _parse_hls(self, value: Selector) -> str:
        value1 = value.css("#video")
        value2 = value1.attrib["data-parameters"]
        value3 = value2.replace("\\", "")
        value4 = value3.replace("&quot;", '"')
        return re.search('"hls":"{"src":"(https?.*?\.m3u8)"', value4)[1]

    def _parse_dash(self, value: Selector) -> str:
        value1 = value.css("#video")
        value2 = value1.attrib["data-parameters"]
        value3 = value2.replace("\\", "")
        value4 = value3.replace("&quot;", '"')
        return re.search('"dash":"{"src":"(https?.*?\.(?:mpd|m3u8))"', value4)[1]

    def parse(self) -> T_AniboomPage:
        return {
            "data_parameters": self._parse_data_parameters(self._doc),
            "hls": self._parse_hls(self._doc),
            "dash": self._parse_dash(self._doc),
        }
